@@ -2,10 +2,10 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2016 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2015 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
-   | that is bundled with this package in the file LICENSE, and is        | 
+   | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
    | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
@@ -36,6 +36,8 @@
 #include "zend_modules.h"
 #include "zend_float.h"
 #include "zend_multibyte.h"
+#include "zend_multiply.h"
+#include "zend_arena.h"
 
 /* Define ZTS if you want a thread-safe Zend */
 /*#undef ZTS*/
@@ -61,31 +63,16 @@ END_EXTERN_C()
 #define ZEND_EARLY_BINDING_DELAYED      1
 #define ZEND_EARLY_BINDING_DELAYED_ALL  2
 
-typedef struct _zend_declarables {
-	zval ticks;
-} zend_declarables;
-
 typedef struct _zend_vm_stack *zend_vm_stack;
 typedef struct _zend_ini_entry zend_ini_entry;
 
 
 struct _zend_compiler_globals {
-	zend_stack bp_stack;
-	zend_stack switch_cond_stack;
-	zend_stack foreach_copy_stack;
-	zend_stack object_stack;
-	zend_stack declare_stack;
+	zend_stack loop_var_stack;
 
 	zend_class_entry *active_class_entry;
 
-	/* variables for list() compilation */
-	zend_llist list_llist;
-	zend_llist dimension_llist;
-	zend_stack list_stack;
-
-	zend_stack function_call_stack;
-
-	char *compiled_filename;
+	zend_string *compiled_filename;
 
 	int zend_lineno;
 
@@ -101,9 +88,6 @@ struct _zend_compiler_globals {
 	zend_bool parse_error;
 	zend_bool in_compilation;
 	zend_bool short_tags;
-	zend_bool asp_tags;
-
-	zend_declarables declarables;
 
 	zend_bool unclean_shutdown;
 
@@ -111,44 +95,24 @@ struct _zend_compiler_globals {
 
 	zend_llist open_files;
 
-	long catch_begin;
-
 	struct _zend_ini_parser_param *ini_parser_param;
 
-	int interactive;
-
-	zend_uint start_lineno;
+	uint32_t start_lineno;
 	zend_bool increment_lineno;
 
-	znode implementing_class;
+	zend_string *doc_comment;
 
-	zend_uint access_type;
-
-	char *doc_comment;
-	zend_uint doc_comment_len;
-
-	zend_uint compiler_options; /* set of ZEND_COMPILE_* constants */
-
-	zval      *current_namespace;
-	HashTable *current_import;
-	HashTable *current_import_function;
-	HashTable *current_import_const;
-	zend_bool  in_namespace;
-	zend_bool  has_bracketed_namespaces;
+	uint32_t compiler_options; /* set of ZEND_COMPILE_* constants */
 
 	HashTable const_filenames;
 
-	zend_compiler_context context;
-	zend_stack context_stack;
+	zend_oparray_context context;
+	zend_file_context file_context;
 
-	/* interned strings */
-	char *interned_strings_start;
-	char *interned_strings_end;
-	char *interned_strings_top;
-	char *interned_strings_snapshot_top;
-#ifndef ZTS
-	char *interned_empty_string;
-#endif
+	zend_arena *arena;
+
+	zend_string *empty_string;
+	zend_string *one_char_string[256];
 
 	HashTable interned_strings;
 
@@ -158,56 +122,51 @@ struct _zend_compiler_globals {
 	zend_bool detect_unicode;
 	zend_bool encoding_declared;
 
+	zend_ast *ast;
+	zend_arena *ast_arena;
+
+	zend_stack delayed_oplines_stack;
+
 #ifdef ZTS
-	zval ***static_members_table;
+	zval **static_members_table;
 	int last_static_member;
 #endif
 };
 
 
 struct _zend_executor_globals {
-	zval **return_value_ptr_ptr;
-
 	zval uninitialized_zval;
-	zval *uninitialized_zval_ptr;
-
 	zval error_zval;
-	zval *error_zval_ptr;
 
 	/* symbol table cache */
-	HashTable *symtable_cache[SYMTABLE_CACHE_SIZE];
-	HashTable **symtable_cache_limit;
-	HashTable **symtable_cache_ptr;
+	zend_array *symtable_cache[SYMTABLE_CACHE_SIZE];
+	zend_array **symtable_cache_limit;
+	zend_array **symtable_cache_ptr;
 
-	zend_op **opline_ptr;
-
-	HashTable *active_symbol_table;
-	HashTable symbol_table;		/* main symbol table */
+	zend_array symbol_table;		/* main symbol table */
 
 	HashTable included_files;	/* files already included */
 
 	JMP_BUF *bailout;
 
 	int error_reporting;
-	int orig_error_reporting;
 	int exit_status;
 
-	zend_op_array *active_op_array;
-
 	HashTable *function_table;	/* function symbol table */
 	HashTable *class_table;		/* class table */
 	HashTable *zend_constants;	/* constants table */
 
+	zval          *vm_stack_top;
+	zval          *vm_stack_end;
+	zend_vm_stack  vm_stack;
+
+	struct _zend_execute_data *current_execute_data;
 	zend_class_entry *scope;
-	zend_class_entry *called_scope; /* Scope of the calling class */
 
-	zval *This;
-
-	long precision;
+	zend_long precision;
 
 	int ticks_count;
 
-	zend_bool in_execution;
 	HashTable *in_autoload;
 	zend_function *autoload_func;
 	zend_bool full_tables_cleanup;
@@ -223,47 +182,50 @@ struct _zend_executor_globals {
 	HashTable regular_list;
 	HashTable persistent_list;
 
-	zend_vm_stack argument_stack;
-
 	int user_error_handler_error_reporting;
-	zval *user_error_handler;
-	zval *user_exception_handler;
+	zval user_error_handler;
+	zval user_exception_handler;
 	zend_stack user_error_handlers_error_reporting;
-	zend_ptr_stack user_error_handlers;
-	zend_ptr_stack user_exception_handlers;
+	zend_stack user_error_handlers;
+	zend_stack user_exception_handlers;
 
 	zend_error_handling_t  error_handling;
 	zend_class_entry      *exception_class;
 
 	/* timeout support */
-	int timeout_seconds;
+	zend_long timeout_seconds;
 
 	int lambda_count;
 
 	HashTable *ini_directives;
 	HashTable *modified_ini_directives;
-	zend_ini_entry *error_reporting_ini_entry;	                
+	zend_ini_entry *error_reporting_ini_entry;
 
 	zend_objects_store objects_store;
-	zval *exception, *prev_exception;
-	zend_op *opline_before_exception;
+	zend_object *exception, *prev_exception;
+	const zend_op *opline_before_exception;
 	zend_op exception_op[3];
 
-	struct _zend_execute_data *current_execute_data;
-
 	struct _zend_module_entry *current_module;
 
-	zend_property_info std_property_info;
+	zend_bool active;
+	zend_bool valid_symbol_table;
 
-	zend_bool active; 
+	zend_long assertions;
 
-	zend_op *start_op;
+	uint32_t           ht_iterators_count;     /* number of allocatd slots */
+	uint32_t           ht_iterators_used;      /* number of used slots */
+	HashTableIterator *ht_iterators;
+	HashTableIterator  ht_iterators_slots[16];
 
 	void *saved_fpu_cw_ptr;
 #if XPFPA_HAVE_CW
 	XPFPA_CW_DATATYPE saved_fpu_cw;
 #endif
 
+	zend_function trampoline;
+	zend_op       call_trampoline_op;
+
 	void *reserved[ZEND_MAX_RESERVED_RESOURCES];
 };
 
@@ -287,6 +249,12 @@ struct _zend_ini_scanner_globals {
 	int scanner_mode;
 };
 
+typedef enum {
+	ON_TOKEN,
+	ON_FEEDBACK,
+	ON_STOP
+} zend_php_scanner_event;
+
 struct _zend_php_scanner_globals {
 	zend_file_handle *yy_in;
 	zend_file_handle *yy_out;
@@ -300,7 +268,7 @@ struct _zend_php_scanner_globals {
 	int yy_state;
 	zend_stack state_stack;
 	zend_ptr_stack heredoc_label_stack;
-	
+
 	/* original (unfiltered) script */
 	unsigned char *script_org;
 	size_t script_org_size;
@@ -313,6 +281,12 @@ struct _zend_php_scanner_globals {
 	zend_encoding_filter input_filter;
 	zend_encoding_filter output_filter;
 	const zend_encoding *script_encoding;
+
+	/* initial string length after scanning to first variable */
+	int scanned_string_len;
+
+	/* hooks */
+	void (* on_event)(zend_php_scanner_event event, int token, int line);
 };
 
 #endif /* ZEND_GLOBALS_H */
