@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2014 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2015 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <zend.h>
 
-const char *zend_vm_opcodes_map[168] = {
+const char *zend_vm_opcodes_map[173] = {
 	"ZEND_NOP",
 	"ZEND_ADD",
 	"ZEND_SUB",
@@ -63,7 +63,7 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_ASSIGN",
 	"ZEND_ASSIGN_REF",
 	"ZEND_ECHO",
-	"ZEND_PRINT",
+	NULL,
 	"ZEND_JMP",
 	"ZEND_JMPZ",
 	"ZEND_JMPNZ",
@@ -71,24 +71,24 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_JMPZ_EX",
 	"ZEND_JMPNZ_EX",
 	"ZEND_CASE",
-	"ZEND_SWITCH_FREE",
-	"ZEND_BRK",
-	"ZEND_CONT",
+	NULL,
+	NULL,
+	NULL,
 	"ZEND_BOOL",
-	"ZEND_INIT_STRING",
-	"ZEND_ADD_CHAR",
-	"ZEND_ADD_STRING",
-	"ZEND_ADD_VAR",
+	"ZEND_FAST_CONCAT",
+	"ZEND_ROPE_INIT",
+	"ZEND_ROPE_ADD",
+	"ZEND_ROPE_END",
 	"ZEND_BEGIN_SILENCE",
 	"ZEND_END_SILENCE",
 	"ZEND_INIT_FCALL_BY_NAME",
 	"ZEND_DO_FCALL",
-	"ZEND_DO_FCALL_BY_NAME",
+	"ZEND_INIT_FCALL",
 	"ZEND_RETURN",
 	"ZEND_RECV",
 	"ZEND_RECV_INIT",
 	"ZEND_SEND_VAL",
-	"ZEND_SEND_VAR",
+	"ZEND_SEND_VAR_EX",
 	"ZEND_SEND_REF",
 	"ZEND_NEW",
 	"ZEND_INIT_NS_FCALL_BY_NAME",
@@ -99,8 +99,8 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_UNSET_VAR",
 	"ZEND_UNSET_DIM",
 	"ZEND_UNSET_OBJ",
-	"ZEND_FE_RESET",
-	"ZEND_FE_FETCH",
+	"ZEND_FE_RESET_R",
+	"ZEND_FE_FETCH_R",
 	"ZEND_EXIT",
 	"ZEND_FETCH_R",
 	"ZEND_FETCH_DIM_R",
@@ -120,9 +120,9 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_FETCH_UNSET",
 	"ZEND_FETCH_DIM_UNSET",
 	"ZEND_FETCH_OBJ_UNSET",
-	"ZEND_FETCH_DIM_TMP_VAR",
+	"ZEND_FETCH_LIST",
 	"ZEND_FETCH_CONSTANT",
-	"ZEND_GOTO",
+	NULL,
 	"ZEND_EXT_STMT",
 	"ZEND_EXT_FCALL_BEGIN",
 	"ZEND_EXT_FCALL_END",
@@ -138,33 +138,33 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_INIT_STATIC_METHOD_CALL",
 	"ZEND_ISSET_ISEMPTY_VAR",
 	"ZEND_ISSET_ISEMPTY_DIM_OBJ",
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
+	"ZEND_SEND_VAL_EX",
+	"ZEND_SEND_VAR",
+	"ZEND_INIT_USER_CALL",
+	"ZEND_SEND_ARRAY",
+	"ZEND_SEND_USER",
+	"ZEND_STRLEN",
+	"ZEND_DEFINED",
+	"ZEND_TYPE_CHECK",
+	"ZEND_VERIFY_RETURN_TYPE",
+	"ZEND_FE_RESET_RW",
+	"ZEND_FE_FETCH_RW",
+	"ZEND_FE_FREE",
+	"ZEND_INIT_DYNAMIC_CALL",
+	"ZEND_DO_ICALL",
+	"ZEND_DO_UCALL",
+	"ZEND_DO_FCALL_BY_NAME",
 	"ZEND_PRE_INC_OBJ",
 	"ZEND_PRE_DEC_OBJ",
 	"ZEND_POST_INC_OBJ",
 	"ZEND_POST_DEC_OBJ",
 	"ZEND_ASSIGN_OBJ",
-	NULL,
+	"ZEND_OP_DATA",
 	"ZEND_INSTANCEOF",
 	"ZEND_DECLARE_CLASS",
 	"ZEND_DECLARE_INHERITED_CLASS",
 	"ZEND_DECLARE_FUNCTION",
-	"ZEND_RAISE_ABSTRACT_ERROR",
+	"ZEND_YIELD_FROM",
 	"ZEND_DECLARE_CONST",
 	"ZEND_ADD_INTERFACE",
 	"ZEND_DECLARE_INHERITED_CLASS_DELAYED",
@@ -173,14 +173,14 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_ISSET_ISEMPTY_PROP_OBJ",
 	"ZEND_HANDLE_EXCEPTION",
 	"ZEND_USER_OPCODE",
-	NULL,
+	"ZEND_ASSERT_CHECK",
 	"ZEND_JMP_SET",
 	"ZEND_DECLARE_LAMBDA_FUNCTION",
 	"ZEND_ADD_TRAIT",
 	"ZEND_BIND_TRAITS",
 	"ZEND_SEPARATE",
-	"ZEND_QM_ASSIGN_VAR",
-	"ZEND_JMP_SET_VAR",
+	"ZEND_FETCH_CLASS_NAME",
+	"ZEND_CALL_TRAMPOLINE",
 	"ZEND_DISCARD_EXCEPTION",
 	"ZEND_YIELD",
 	"ZEND_GENERATOR_RETURN",
@@ -190,6 +190,11 @@ const char *zend_vm_opcodes_map[168] = {
 	"ZEND_SEND_UNPACK",
 	"ZEND_POW",
 	"ZEND_ASSIGN_POW",
+	"ZEND_BIND_GLOBAL",
+	"ZEND_COALESCE",
+	"ZEND_SPACESHIP",
+	"ZEND_DECLARE_ANON_CLASS",
+	"ZEND_DECLARE_ANON_INHERITED_CLASS",
 };
 
 ZEND_API const char* zend_get_opcode_name(zend_uchar opcode) {
