@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
@@ -28,123 +28,111 @@
 #include "php_dom.h"
 
 /*
-* class DOMDocumentType extends DOMNode 
+* class DOMDocumentType extends DOMNode
 *
 * URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-412266927
-* Since: 
+* Since:
 */
 
 const zend_function_entry php_dom_documenttype_class_functions[] = {
 	PHP_FE_END
 };
 
-/* {{{ name	string	
-readonly=yes 
+/* {{{ name	string
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1844763134
-Since: 
+Since:
 */
-int dom_documenttype_name_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_name_read(dom_object *obj, zval *retval)
 {
-	xmlDtdPtr dtdptr;
-
-	dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
+	xmlDtdPtr dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
 
 	if (dtdptr == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
-	ZVAL_STRING(*retval, (char *) (dtdptr->name), 1);
+	ZVAL_STRING(retval, (char *) (dtdptr->name));
 
 	return SUCCESS;
 }
 
 /* }}} */
 
-/* {{{ entities	DOMNamedNodeMap	
-readonly=yes 
+/* {{{ entities	DOMNamedNodeMap
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1788794630
-Since: 
+Since:
 */
-int dom_documenttype_entities_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_entities_read(dom_object *obj, zval *retval)
 {
-	xmlDtdPtr doctypep;
+	xmlDtdPtr doctypep = (xmlDtdPtr) dom_object_get_node(obj);
 	xmlHashTable *entityht;
 	dom_object *intern;
 
-	doctypep = (xmlDtdPtr) dom_object_get_node(obj);
-
 	if (doctypep == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	MAKE_STD_ZVAL(*retval);
-	php_dom_create_interator(*retval, DOM_NAMEDNODEMAP TSRMLS_CC);
+	php_dom_create_interator(retval, DOM_NAMEDNODEMAP);
 
 	entityht = (xmlHashTable *) doctypep->entities;
 
-	intern = (dom_object *)zend_objects_get_address(*retval TSRMLS_CC);
-	dom_namednode_iter(obj, XML_ENTITY_NODE, intern, entityht, NULL, NULL TSRMLS_CC);
+	intern = Z_DOMOBJ_P(retval);
+	dom_namednode_iter(obj, XML_ENTITY_NODE, intern, entityht, NULL, NULL);
 
 	return SUCCESS;
 }
 
 /* }}} */
 
-/* {{{ notations	DOMNamedNodeMap	
-readonly=yes 
+/* {{{ notations	DOMNamedNodeMap
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-D46829EF
-Since: 
+Since:
 */
-int dom_documenttype_notations_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_notations_read(dom_object *obj, zval *retval)
 {
-	xmlDtdPtr doctypep;
+	xmlDtdPtr doctypep = (xmlDtdPtr) dom_object_get_node(obj);
 	xmlHashTable *notationht;
 	dom_object *intern;
 
-	doctypep = (xmlDtdPtr) dom_object_get_node(obj);
-
 	if (doctypep == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	MAKE_STD_ZVAL(*retval);
-	php_dom_create_interator(*retval, DOM_NAMEDNODEMAP TSRMLS_CC);
+	php_dom_create_interator(retval, DOM_NAMEDNODEMAP);
 
 	notationht = (xmlHashTable *) doctypep->notations;
 
-	intern = (dom_object *)zend_objects_get_address(*retval TSRMLS_CC);
-	dom_namednode_iter(obj, XML_NOTATION_NODE, intern, notationht, NULL, NULL TSRMLS_CC);
+	intern = Z_DOMOBJ_P(retval);
+	dom_namednode_iter(obj, XML_NOTATION_NODE, intern, notationht, NULL, NULL);
 
 	return SUCCESS;
 }
 
 /* }}} */
 
-/* {{{ publicId	string	
-readonly=yes 
+/* {{{ publicId	string
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-Core-DocType-publicId
 Since: DOM Level 2
 */
-int dom_documenttype_public_id_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_public_id_read(dom_object *obj, zval *retval)
 {
-	xmlDtdPtr dtdptr;
-
-	dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
+	xmlDtdPtr dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
 
 	if (dtdptr == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
 	if (dtdptr->ExternalID) {
-		ZVAL_STRING(*retval, (char *) (dtdptr->ExternalID), 1);
+		ZVAL_STRING(retval, (char *) (dtdptr->ExternalID));
 	} else {
-		ZVAL_EMPTY_STRING(*retval);
+		ZVAL_EMPTY_STRING(retval);
 	}
 	return SUCCESS;
 
@@ -152,54 +140,46 @@ int dom_documenttype_public_id_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-/* {{{ systemId	string	
-readonly=yes 
+/* {{{ systemId	string
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-Core-DocType-systemId
 Since: DOM Level 2
 */
-int dom_documenttype_system_id_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_system_id_read(dom_object *obj, zval *retval)
 {
-	xmlDtdPtr dtdptr;
-
-	dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
+	xmlDtdPtr dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
 
 	if (dtdptr == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
 	if (dtdptr->SystemID) {
-		ZVAL_STRING(*retval, (char *) (dtdptr->SystemID), 1);
+		ZVAL_STRING(retval, (char *) (dtdptr->SystemID));
 	} else {
-		ZVAL_EMPTY_STRING(*retval);
+		ZVAL_EMPTY_STRING(retval);
 	}
 	return SUCCESS;
 }
 
 /* }}} */
 
-/* {{{ internalSubset	string	
-readonly=yes 
+/* {{{ internalSubset	string
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-Core-DocType-internalSubset
 Since: DOM Level 2
 */
-int dom_documenttype_internal_subset_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_documenttype_internal_subset_read(dom_object *obj, zval *retval)
 {
-
-	xmlDtdPtr dtdptr;
+	xmlDtdPtr dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
 	xmlDtdPtr intsubset;
 
-	dtdptr = (xmlDtdPtr) dom_object_get_node(obj);
-
 	if (dtdptr == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
-
-	if (dtdptr->doc != NULL && ((intsubset = xmlGetIntSubset(dtdptr->doc)) != NULL) && intsubset->children != NULL) {
+	if (dtdptr->doc != NULL && ((intsubset = xmlGetIntSubset(dtdptr->doc)) != NULL)) {
 		smart_str ret_buf = {0};
 		xmlNodePtr cur = intsubset->children;
 
@@ -211,9 +191,9 @@ int dom_documenttype_internal_subset_read(dom_object *obj, zval **retval TSRMLS_
 				xmlOutputBufferFlush(buff);
 
 #ifdef LIBXML2_NEW_BUFFER
-				smart_str_appendl(&ret_buf, xmlOutputBufferGetContent(buff), xmlOutputBufferGetSize(buff));
+				smart_str_appendl(&ret_buf, (const char *) xmlOutputBufferGetContent(buff), xmlOutputBufferGetSize(buff));
 #else
-				smart_str_appendl(&ret_buf, buff->buffer->content, buff->buffer->use);
+				smart_str_appendl(&ret_buf, (char *) buff->buffer->content, buff->buffer->use);
 #endif
 
 				(void)xmlOutputBufferClose(buff);
@@ -222,14 +202,14 @@ int dom_documenttype_internal_subset_read(dom_object *obj, zval **retval TSRMLS_
 			cur = cur->next;
 		}
 
-		if (ret_buf.len) {
-			ZVAL_STRINGL(*retval, ret_buf.c, ret_buf.len, 1);
-			smart_str_free(&ret_buf);
+		if (ret_buf.s) {
+			smart_str_0(&ret_buf);
+			ZVAL_NEW_STR(retval, ret_buf.s);
 			return SUCCESS;
 		}
 	}
 
-	ZVAL_NULL(*retval);
+	ZVAL_NULL(retval);
 
 	return SUCCESS;
 
