@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -120,10 +120,10 @@ ZEND_END_ARG_INFO();
 /* }}} */
 
 /*
-* class DOMElement extends DOMNode 
+* class DOMElement extends DOMNode
 *
 * URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-745549614
-* Since: 
+* Since:
 */
 
 const zend_function_entry php_dom_element_class_functions[] = { /* {{{ */
@@ -150,30 +150,27 @@ const zend_function_entry php_dom_element_class_functions[] = { /* {{{ */
 };
 /* }}} */
 
-/* {{{ proto void DOMElement::__construct(string name, [string value], [string uri]); */
+/* {{{ proto void DOMElement::__construct(string name, [string value], [string uri]) */
 PHP_METHOD(domelement, __construct)
 {
 
-	zval *id;
+	zval *id = getThis();
 	xmlNodePtr nodep = NULL, oldnode = NULL;
 	dom_object *intern;
 	char *name, *value = NULL, *uri = NULL;
 	char *localname = NULL, *prefix = NULL;
-	int errorcode = 0, uri_len = 0;
-	int name_len, value_len = 0, name_valid;
+	int errorcode = 0;
+	size_t name_len, value_len = 0, uri_len = 0;
+	int name_valid;
 	xmlNsPtr nsptr = NULL;
-	zend_error_handling error_handling;
 
-	zend_replace_error_handling(EH_THROW, dom_domexception_class_entry, &error_handling TSRMLS_CC);
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|s!s", &id, dom_element_class_entry, &name, &name_len, &value, &value_len, &uri, &uri_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	if (zend_parse_parameters_throw(ZEND_NUM_ARGS(), "s|s!s", &name, &name_len, &value, &value_len, &uri, &uri_len) == FAILURE) {
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
 
 	name_valid = xmlValidateName((xmlChar *) name, 0);
 	if (name_valid != 0) {
-		php_dom_throw_error(INVALID_CHARACTER_ERR, 1 TSRMLS_CC);
+		php_dom_throw_error(INVALID_CHARACTER_ERR, 1);
 		RETURN_FALSE;
 	}
 
@@ -195,23 +192,23 @@ PHP_METHOD(domelement, __construct)
 			if (nodep != NULL) {
 				xmlFreeNode(nodep);
 			}
-			php_dom_throw_error(errorcode, 1 TSRMLS_CC);
+			php_dom_throw_error(errorcode, 1);
 			RETURN_FALSE;
 		}
 	} else {
 	    /* If you don't pass a namespace uri, then you can't set a prefix */
-	    localname = xmlSplitQName2((xmlChar *)name, (xmlChar **) &prefix);
+	    localname = (char *) xmlSplitQName2((xmlChar *) name, (xmlChar **) &prefix);
 	    if (prefix != NULL) {
 			xmlFree(localname);
 			xmlFree(prefix);
-	        php_dom_throw_error(NAMESPACE_ERR, 1 TSRMLS_CC);
+	        php_dom_throw_error(NAMESPACE_ERR, 1);
 	        RETURN_FALSE;
 	    }
 		nodep = xmlNewNode(NULL, (xmlChar *) name);
 	}
 
 	if (!nodep) {
-		php_dom_throw_error(INVALID_STATE_ERR, 1 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 1);
 		RETURN_FALSE;
 	}
 
@@ -219,23 +216,21 @@ PHP_METHOD(domelement, __construct)
 		xmlNodeSetContentLen(nodep, (xmlChar *) value, value_len);
 	}
 
-	intern = (dom_object *)zend_object_store_get_object(id TSRMLS_CC);
-	if (intern != NULL) {
-		oldnode = dom_object_get_node(intern);
-		if (oldnode != NULL) {
-			php_libxml_node_free_resource(oldnode  TSRMLS_CC);
-		}
-		php_libxml_increment_node_ptr((php_libxml_node_object *)intern, nodep, (void *)intern TSRMLS_CC);
+	intern = Z_DOMOBJ_P(id);
+	oldnode = dom_object_get_node(intern);
+	if (oldnode != NULL) {
+		php_libxml_node_free_resource(oldnode );
 	}
+	php_libxml_increment_node_ptr((php_libxml_node_object *)intern, nodep, (void *)intern);
 }
 /* }}} end DOMElement::__construct */
 
-/* {{{ tagName	string	
-readonly=yes 
+/* {{{ tagName	string
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-104682815
-Since: 
+Since:
 */
-int dom_element_tag_name_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_element_tag_name_read(dom_object *obj, zval *retval)
 {
 	xmlNodePtr nodep;
 	xmlNsPtr ns;
@@ -244,20 +239,19 @@ int dom_element_tag_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 	nodep = dom_object_get_node(obj);
 
 	if (nodep == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 0 TSRMLS_CC);
+		php_dom_throw_error(INVALID_STATE_ERR, 0);
 		return FAILURE;
 	}
 
-	ALLOC_ZVAL(*retval);
 	ns = nodep->ns;
 	if (ns != NULL && ns->prefix) {
 		qname = xmlStrdup(ns->prefix);
 		qname = xmlStrcat(qname, (xmlChar *)":");
 		qname = xmlStrcat(qname, nodep->name);
-		ZVAL_STRING(*retval, (char *)qname, 1);
+		ZVAL_STRING(retval, (char *)qname);
 		xmlFree(qname);
 	} else {
-		ZVAL_STRING(*retval, (char *) nodep->name, 1);
+		ZVAL_STRING(retval, (char *) nodep->name);
 	}
 
 	return SUCCESS;
@@ -265,15 +259,14 @@ int dom_element_tag_name_read(dom_object *obj, zval **retval TSRMLS_DC)
 
 /* }}} */
 
-/* {{{ schemaTypeInfo	typeinfo	
-readonly=yes 
+/* {{{ schemaTypeInfo	typeinfo
+readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#Element-schemaTypeInfo
 Since: DOM Level 3
 */
-int dom_element_schema_type_info_read(dom_object *obj, zval **retval TSRMLS_DC)
+int dom_element_schema_type_info_read(dom_object *obj, zval *retval)
 {
-	ALLOC_ZVAL(*retval);
-	ZVAL_NULL(*retval);
+	ZVAL_NULL(retval);
 	return SUCCESS;
 }
 
@@ -334,9 +327,9 @@ PHP_FUNCTION(dom_element_get_attribute)
 	xmlChar *value = NULL;
 	dom_object *intern;
 	xmlNodePtr attr;
-	int name_len;
+	size_t name_len;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -355,11 +348,11 @@ PHP_FUNCTION(dom_element_get_attribute)
 				value = xmlStrdup(((xmlAttributePtr)attr)->defaultValue);
 		}
 	}
-	
+
 	if (value == NULL) {
 		RETURN_EMPTY_STRING();
 	} else {
-		RETVAL_STRING((char *)value, 1);
+		RETVAL_STRING((char *)value);
 		xmlFree(value);
 	}
 }
@@ -367,36 +360,37 @@ PHP_FUNCTION(dom_element_get_attribute)
 
 /* {{{ proto void dom_element_set_attribute(string name, string value);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-F68F082
-Since: 
+Since:
 */
 PHP_FUNCTION(dom_element_set_attribute)
 {
 	zval *id;
 	xmlNode *nodep;
 	xmlNodePtr attr = NULL;
-	int ret, name_len, value_len, name_valid;
+	int ret, name_valid;
+	size_t name_len, value_len;
 	dom_object *intern;
 	char *name, *value;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oss", &id, dom_element_class_entry, &name, &name_len, &value, &value_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Oss", &id, dom_element_class_entry, &name, &name_len, &value, &value_len) == FAILURE) {
 		return;
 	}
 
 	if (name_len == 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Attribute Name is required");
+		php_error_docref(NULL, E_WARNING, "Attribute Name is required");
 		RETURN_FALSE;
 	}
 
 	name_valid = xmlValidateName((xmlChar *) name, 0);
 	if (name_valid != 0) {
-		php_dom_throw_error(INVALID_CHARACTER_ERR, 1 TSRMLS_CC);
+		php_dom_throw_error(INVALID_CHARACTER_ERR, 1);
 		RETURN_FALSE;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
@@ -404,7 +398,7 @@ PHP_FUNCTION(dom_element_set_attribute)
 	if (attr != NULL) {
 		switch (attr->type) {
 			case XML_ATTRIBUTE_NODE:
-				node_list_unlink(attr->children TSRMLS_CC);
+				node_list_unlink(attr->children);
 				break;
 			case XML_NAMESPACE_DECL:
 				RETURN_FALSE;
@@ -422,7 +416,7 @@ PHP_FUNCTION(dom_element_set_attribute)
 		attr = (xmlNodePtr)xmlSetProp(nodep, (xmlChar *) name, (xmlChar *)value);
 	}
 	if (!attr) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "No such attribute '%s'", name);
+		php_error_docref(NULL, E_WARNING, "No such attribute '%s'", name);
 		RETURN_FALSE;
 	}
 
@@ -440,17 +434,17 @@ PHP_FUNCTION(dom_element_remove_attribute)
 	zval *id;
 	xmlNodePtr nodep, attrp;
 	dom_object *intern;
-	int name_len;
+	size_t name_len;
 	char *name;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
@@ -462,7 +456,7 @@ PHP_FUNCTION(dom_element_remove_attribute)
 	switch (attrp->type) {
 		case XML_ATTRIBUTE_NODE:
 			if (php_dom_object_get_data(attrp) == NULL) {
-			node_list_unlink(attrp->children TSRMLS_CC);
+			node_list_unlink(attrp->children);
 				xmlUnlinkNode(attrp);
 				xmlFreeProp((xmlAttrPtr)attrp);
 		} else {
@@ -481,17 +475,18 @@ PHP_FUNCTION(dom_element_remove_attribute)
 
 /* {{{ proto DOMAttr dom_element_get_attribute_node(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-217A91B8
-Since: 
+Since:
 */
 PHP_FUNCTION(dom_element_get_attribute_node)
 {
 	zval *id;
 	xmlNodePtr nodep, attrp;
-	int name_len, ret;
+	size_t name_len;
+	int ret;
 	dom_object *intern;
 	char *name;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -527,7 +522,7 @@ PHP_FUNCTION(dom_element_get_attribute_node)
 
 /* {{{ proto DOMAttr dom_element_set_attribute_node(DOMAttr newAttr);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-887236154
-Since: 
+Since:
 */
 PHP_FUNCTION(dom_element_set_attribute_node)
 {
@@ -537,32 +532,32 @@ PHP_FUNCTION(dom_element_set_attribute_node)
 	dom_object *intern, *attrobj, *oldobj;
 	int ret;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
 	DOM_GET_OBJ(attrp, node, xmlAttrPtr, attrobj);
 
 	if (attrp->type != XML_ATTRIBUTE_NODE) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Attribute node is required");
+		php_error_docref(NULL, E_WARNING, "Attribute node is required");
 		RETURN_FALSE;
 	}
 
 	if (!(attrp->doc == NULL || attrp->doc == nodep->doc)) {
-		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
 	existattrp = xmlHasProp(nodep, attrp->name);
 	if (existattrp != NULL && existattrp->type != XML_ATTRIBUTE_DECL) {
-		if ((oldobj = php_dom_object_get_data((xmlNodePtr) existattrp)) != NULL && 
+		if ((oldobj = php_dom_object_get_data((xmlNodePtr) existattrp)) != NULL &&
 			((php_libxml_node_ptr *)oldobj->ptr)->node == (xmlNodePtr) attrp)
 		{
 			RETURN_NULL();
@@ -576,7 +571,7 @@ PHP_FUNCTION(dom_element_set_attribute_node)
 
 	if (attrp->doc == NULL && nodep->doc != NULL) {
 		attrobj->document = intern->document;
-		php_libxml_increment_doc_ref((php_libxml_node_object *)attrobj, NULL TSRMLS_CC);
+		php_libxml_increment_doc_ref((php_libxml_node_object *)attrobj, NULL);
 	}
 
 	xmlAddChild(nodep, (xmlNodePtr) attrp);
@@ -593,7 +588,7 @@ PHP_FUNCTION(dom_element_set_attribute_node)
 
 /* {{{ proto DOMAttr dom_element_remove_attribute_node(DOMAttr oldAttr);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-D589198
-Since: 
+Since:
 */
 PHP_FUNCTION(dom_element_remove_attribute_node)
 {
@@ -603,21 +598,21 @@ PHP_FUNCTION(dom_element_remove_attribute_node)
 	dom_object *intern, *attrobj;
 	int ret;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
 	DOM_GET_OBJ(attrp, node, xmlAttrPtr, attrobj);
 
 	if (attrp->type != XML_ATTRIBUTE_NODE || attrp->parent != nodep) {
-		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
@@ -630,27 +625,27 @@ PHP_FUNCTION(dom_element_remove_attribute_node)
 
 /* {{{ proto DOMNodeList dom_element_get_elements_by_tag_name(string name);
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-1938918D
-Since: 
+Since:
 */
 PHP_FUNCTION(dom_element_get_elements_by_tag_name)
 {
 	zval *id;
 	xmlNodePtr elemp;
-	int name_len;
+	size_t name_len;
 	dom_object *intern, *namednode;
 	char *name;
 	xmlChar *local;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(elemp, id, xmlNodePtr, intern);
 
-	php_dom_create_interator(return_value, DOM_NODELIST TSRMLS_CC);
-	namednode = (dom_object *)zend_objects_get_address(return_value TSRMLS_CC);
+	php_dom_create_interator(return_value, DOM_NODELIST);
+	namednode = Z_DOMOBJ_P(return_value);
 	local = xmlCharStrndup(name, name_len);
-	dom_namednode_iter(intern, 0, namednode, NULL, local, NULL TSRMLS_CC);
+	dom_namednode_iter(intern, 0, namednode, NULL, local, NULL);
 }
 /* }}} end dom_element_get_elements_by_tag_name */
 
@@ -664,11 +659,11 @@ PHP_FUNCTION(dom_element_get_attribute_ns)
 	xmlNodePtr elemp;
 	xmlNsPtr nsptr;
 	dom_object *intern;
-	int uri_len = 0, name_len = 0;
+	size_t uri_len = 0, name_len = 0;
 	char *uri, *name;
 	xmlChar *strattr;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -677,13 +672,13 @@ PHP_FUNCTION(dom_element_get_attribute_ns)
 	strattr = xmlGetNsProp(elemp, (xmlChar *) name, (xmlChar *) uri);
 
 	if (strattr != NULL) {
-		RETVAL_STRING((char *)strattr, 1);
+		RETVAL_STRING((char *)strattr);
 		xmlFree(strattr);
 	} else {
 		if (xmlStrEqual((xmlChar *) uri, (xmlChar *)DOM_XMLNS_NAMESPACE)) {
 			nsptr = dom_get_nsdecl(elemp, (xmlChar *)name);
 			if (nsptr != NULL) {
-				RETVAL_STRING((char *) nsptr->href, 1);
+				RETVAL_STRING((char *) nsptr->href);
 			} else {
 				RETVAL_EMPTY_STRING();
 			}
@@ -721,7 +716,7 @@ static xmlNsPtr _dom_new_reconNs(xmlDocPtr doc, xmlNodePtr tree, xmlNsPtr ns) /*
 		if (ns->prefix == NULL)
 			snprintf((char *) prefix, sizeof(prefix), "default%d", counter++);
 		else
-			snprintf((char *) prefix, sizeof(prefix), "%.20s%d", 
+			snprintf((char *) prefix, sizeof(prefix), "%.20s%d",
 			(char *)ns->prefix, counter++);
 		def = xmlSearchNs(doc, tree, prefix);
 	}
@@ -744,18 +739,18 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 	xmlNodePtr elemp, nodep = NULL;
 	xmlNsPtr nsptr;
 	xmlAttr *attr;
-	int uri_len = 0, name_len = 0, value_len = 0;
+	size_t uri_len = 0, name_len = 0, value_len = 0;
 	char *uri, *name, *value;
 	char *localname = NULL, *prefix = NULL;
 	dom_object *intern;
 	int errorcode = 0, stricterror, is_xmlns = 0, name_valid;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!ss", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len, &value, &value_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os!ss", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len, &value, &value_len) == FAILURE) {
 		return;
 	}
 
 	if (name_len == 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Attribute Name is required");
+		php_error_docref(NULL, E_WARNING, "Attribute Name is required");
 		RETURN_FALSE;
 	}
 
@@ -764,7 +759,7 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 	stricterror = dom_get_strict_error(intern->document);
 
 	if (dom_node_is_read_only(elemp) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, stricterror TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, stricterror);
 		RETURN_NULL();
 	}
 
@@ -774,11 +769,11 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 		if (uri_len > 0) {
 			nodep = (xmlNodePtr) xmlHasNsProp(elemp, (xmlChar *) localname, (xmlChar *) uri);
 			if (nodep != NULL && nodep->type != XML_ATTRIBUTE_DECL) {
-				node_list_unlink(nodep->children TSRMLS_CC);
+				node_list_unlink(nodep->children);
 			}
 
-			if ((xmlStrEqual((xmlChar *) prefix, (xmlChar *)"xmlns") || 
-				(prefix == NULL && xmlStrEqual((xmlChar *) localname, (xmlChar *)"xmlns"))) && 
+			if ((xmlStrEqual((xmlChar *) prefix, (xmlChar *)"xmlns") ||
+				(prefix == NULL && xmlStrEqual((xmlChar *) localname, (xmlChar *)"xmlns"))) &&
 				xmlStrEqual((xmlChar *) uri, (xmlChar *)DOM_XMLNS_NAMESPACE)) {
 				is_xmlns = 1;
 				if (prefix == NULL) {
@@ -793,7 +788,7 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 
 					tmpnsptr = nsptr->next;
 					while (tmpnsptr) {
-						if ((tmpnsptr->prefix != NULL) && (tmpnsptr->href != NULL) && 
+						if ((tmpnsptr->prefix != NULL) && (tmpnsptr->href != NULL) &&
 							(xmlStrEqual(tmpnsptr->href, (xmlChar *) uri))) {
 							nsptr = tmpnsptr;
 							break;
@@ -842,7 +837,7 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 			} else {
 				attr = xmlHasProp(elemp, (xmlChar *)localname);
 				if (attr != NULL && attr->type != XML_ATTRIBUTE_DECL) {
-					node_list_unlink(attr->children TSRMLS_CC);
+					node_list_unlink(attr->children);
 				}
 				xmlSetProp(elemp, (xmlChar *)localname, (xmlChar *)value);
 			}
@@ -855,7 +850,7 @@ PHP_FUNCTION(dom_element_set_attribute_ns)
 	}
 
 	if (errorcode != 0) {
-		php_dom_throw_error(errorcode, stricterror TSRMLS_CC);
+		php_dom_throw_error(errorcode, stricterror);
 	}
 
 	RETURN_NULL();
@@ -873,17 +868,17 @@ PHP_FUNCTION(dom_element_remove_attribute_ns)
 	xmlAttr *attrp;
 	xmlNsPtr nsptr;
 	dom_object *intern;
-	int name_len, uri_len;
+	size_t name_len, uri_len;
 	char *name, *uri;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_NULL();
 	}
 
@@ -907,7 +902,7 @@ PHP_FUNCTION(dom_element_remove_attribute_ns)
 
 	if (attrp && attrp->type != XML_ATTRIBUTE_DECL) {
 		if (php_dom_object_get_data((xmlNodePtr) attrp) == NULL) {
-			node_list_unlink(attrp->children TSRMLS_CC);
+			node_list_unlink(attrp->children);
 			xmlUnlinkNode((xmlNodePtr) attrp);
 			xmlFreeProp(attrp);
 		} else {
@@ -929,10 +924,11 @@ PHP_FUNCTION(dom_element_get_attribute_node_ns)
 	xmlNodePtr elemp;
 	xmlAttrPtr attrp;
 	dom_object *intern;
-	int uri_len, name_len, ret;
+	size_t uri_len, name_len;
+	int ret;
 	char *uri, *name;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -962,26 +958,26 @@ PHP_FUNCTION(dom_element_set_attribute_node_ns)
 	dom_object *intern, *attrobj, *oldobj;
 	int ret;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OO", &id, dom_element_class_entry, &node, dom_attr_class_entry) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
 	DOM_GET_OBJ(attrp, node, xmlAttrPtr, attrobj);
 
 	if (attrp->type != XML_ATTRIBUTE_NODE) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Attribute node is required");
+		php_error_docref(NULL, E_WARNING, "Attribute node is required");
 		RETURN_FALSE;
 	}
 
 	if (!(attrp->doc == NULL || attrp->doc == nodep->doc)) {
-		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(WRONG_DOCUMENT_ERR, dom_get_strict_error(intern->document));
 		RETURN_FALSE;
 	}
 
@@ -993,7 +989,7 @@ PHP_FUNCTION(dom_element_set_attribute_node_ns)
     }
 
 	if (existattrp != NULL && existattrp->type != XML_ATTRIBUTE_DECL) {
-		if ((oldobj = php_dom_object_get_data((xmlNodePtr) existattrp)) != NULL && 
+		if ((oldobj = php_dom_object_get_data((xmlNodePtr) existattrp)) != NULL &&
 			((php_libxml_node_ptr *)oldobj->ptr)->node == (xmlNodePtr) attrp)
 		{
 			RETURN_NULL();
@@ -1007,7 +1003,7 @@ PHP_FUNCTION(dom_element_set_attribute_node_ns)
 
 	if (attrp->doc == NULL && nodep->doc != NULL) {
 		attrobj->document = intern->document;
-		php_libxml_increment_doc_ref((php_libxml_node_object *)attrobj, NULL TSRMLS_CC);
+		php_libxml_increment_doc_ref((php_libxml_node_object *)attrobj, NULL);
 	}
 
 	xmlAddChild(nodep, (xmlNodePtr) attrp);
@@ -1030,22 +1026,22 @@ PHP_FUNCTION(dom_element_get_elements_by_tag_name_ns)
 {
 	zval *id;
 	xmlNodePtr elemp;
-	int uri_len, name_len;
+	size_t uri_len, name_len;
 	dom_object *intern, *namednode;
 	char *uri, *name;
 	xmlChar *local, *nsuri;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oss", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Oss", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(elemp, id, xmlNodePtr, intern);
 
-	php_dom_create_interator(return_value, DOM_NODELIST TSRMLS_CC);
-	namednode = (dom_object *)zend_objects_get_address(return_value TSRMLS_CC);
+	php_dom_create_interator(return_value, DOM_NODELIST);
+	namednode = Z_DOMOBJ_P(return_value);
 	local = xmlCharStrndup(name, name_len);
 	nsuri = xmlCharStrndup(uri, uri_len);
-	dom_namednode_iter(intern, 0, namednode, NULL, local, nsuri TSRMLS_CC);
+	dom_namednode_iter(intern, 0, namednode, NULL, local, nsuri);
 
 }
 /* }}} end dom_element_get_elements_by_tag_name_ns */
@@ -1060,10 +1056,10 @@ PHP_FUNCTION(dom_element_has_attribute)
 	xmlNode *nodep;
 	dom_object *intern;
 	char *name;
-	int name_len;
+	size_t name_len;
 	xmlNodePtr attr;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &id, dom_element_class_entry, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -1088,11 +1084,11 @@ PHP_FUNCTION(dom_element_has_attribute_ns)
 	xmlNodePtr elemp;
 	xmlNs *nsp;
 	dom_object *intern;
-	int uri_len, name_len;
+	size_t uri_len, name_len;
 	char *uri, *name;
 	xmlChar *value;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os!s", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len) == FAILURE) {
 		return;
 	}
 
@@ -1146,23 +1142,23 @@ PHP_FUNCTION(dom_element_set_id_attribute)
 	xmlAttrPtr attrp;
 	dom_object *intern;
 	char *name;
-	int name_len;
+	size_t name_len;
 	zend_bool is_id;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Osb", &id, dom_element_class_entry, &name, &name_len, &is_id) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Osb", &id, dom_element_class_entry, &name, &name_len, &is_id) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_NULL();
 	}
 
 	attrp = xmlHasNsProp(nodep, (xmlChar *)name, NULL);
 	if (attrp == NULL || attrp->type == XML_ATTRIBUTE_DECL) {
-		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document));
 	} else {
 		php_set_attribute_id(attrp, is_id);
 	}
@@ -1181,24 +1177,24 @@ PHP_FUNCTION(dom_element_set_id_attribute_ns)
 	xmlNodePtr elemp;
 	xmlAttrPtr attrp;
 	dom_object *intern;
-	int uri_len, name_len;
+	size_t uri_len, name_len;
 	char *uri, *name;
 	zend_bool is_id;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Ossb", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len, &is_id) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Ossb", &id, dom_element_class_entry, &uri, &uri_len, &name, &name_len, &is_id) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(elemp, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(elemp) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_NULL();
 	}
 
 	attrp = xmlHasNsProp(elemp, (xmlChar *)name, (xmlChar *)uri);
 	if (attrp == NULL || attrp->type == XML_ATTRIBUTE_DECL) {
-		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document));
 	} else {
 		php_set_attribute_id(attrp, is_id);
 	}
@@ -1219,21 +1215,21 @@ PHP_FUNCTION(dom_element_set_id_attribute_node)
 	dom_object *intern, *attrobj;
 	zend_bool is_id;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "OOb", &id, dom_element_class_entry, &node, dom_attr_class_entry, &is_id) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OOb", &id, dom_element_class_entry, &node, dom_attr_class_entry, &is_id) == FAILURE) {
 		return;
 	}
 
 	DOM_GET_OBJ(nodep, id, xmlNodePtr, intern);
 
 	if (dom_node_is_read_only(nodep) == SUCCESS) {
-		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NO_MODIFICATION_ALLOWED_ERR, dom_get_strict_error(intern->document));
 		RETURN_NULL();
 	}
 
 	DOM_GET_OBJ(attrp, node, xmlAttrPtr, attrobj);
 
 	if (attrp->parent != nodep) {
-		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document) TSRMLS_CC);
+		php_dom_throw_error(NOT_FOUND_ERR, dom_get_strict_error(intern->document));
 	} else {
 		php_set_attribute_id(attrp, is_id);
 	}
