@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -36,48 +36,36 @@ static zend_object_handlers Collator_handlers;
  */
 
 /* {{{ Collator_objects_dtor */
-static void Collator_objects_dtor(
-	void *object,
-	zend_object_handle handle TSRMLS_DC )
+static void Collator_objects_dtor(zend_object *object )
 {
-	zend_objects_destroy_object( object, handle TSRMLS_CC );
+	zend_objects_destroy_object(object );
 }
 /* }}} */
 
 /* {{{ Collator_objects_free */
-void Collator_objects_free( zend_object *object TSRMLS_DC )
+void Collator_objects_free(zend_object *object )
 {
-	Collator_object* co = (Collator_object*)object;
+	Collator_object* co = php_intl_collator_fetch_object(object);
 
-	zend_object_std_dtor( &co->zo TSRMLS_CC );
+	zend_object_std_dtor(&co->zo );
 
-	collator_object_destroy( co TSRMLS_CC );
-
-	efree( co );
+	collator_object_destroy(co );
 }
 /* }}} */
 
 /* {{{ Collator_object_create */
-zend_object_value Collator_object_create(
-	zend_class_entry *ce TSRMLS_DC )
+zend_object *Collator_object_create(zend_class_entry *ce )
 {
-	zend_object_value    retval;
 	Collator_object*     intern;
 
-	intern = ecalloc( 1, sizeof(Collator_object) );
-	intl_error_init( COLLATOR_ERROR_P( intern ) TSRMLS_CC );
-	zend_object_std_init( &intern->zo, ce TSRMLS_CC );
+	intern = ecalloc(1, sizeof(Collator_object) + zend_object_properties_size(ce));
+	intl_error_init(COLLATOR_ERROR_P(intern));
+	zend_object_std_init(&intern->zo, ce );
 	object_properties_init(&intern->zo, ce);
 
-	retval.handle = zend_objects_store_put(
-		intern,
-		Collator_objects_dtor,
-		(zend_objects_free_object_storage_t)Collator_objects_free,
-		NULL TSRMLS_CC );
+	intern->zo.handlers = &Collator_handlers;
 
-	retval.handlers = &Collator_handlers;
-
-	return retval;
+	return &intern->zo;
 }
 /* }}} */
 
@@ -135,20 +123,23 @@ zend_function_entry Collator_class_functions[] = {
 /* {{{ collator_register_Collator_class
  * Initialize 'Collator' class
  */
-void collator_register_Collator_class( TSRMLS_D )
+void collator_register_Collator_class( void )
 {
 	zend_class_entry ce;
 
 	/* Create and register 'Collator' class. */
 	INIT_CLASS_ENTRY( ce, "Collator", Collator_class_functions );
 	ce.create_object = Collator_object_create;
-	Collator_ce_ptr = zend_register_internal_class( &ce TSRMLS_CC );
+	Collator_ce_ptr = zend_register_internal_class( &ce );
 
 	memcpy(&Collator_handlers, zend_get_std_object_handlers(),
 		sizeof Collator_handlers);
-	/* Collator has no usable clone semantics - ucol_cloneBinary/ucol_openBinary require binary buffer 
-	   for which we don't have the place to keep */	
-	Collator_handlers.clone_obj = NULL; 
+	/* Collator has no usable clone semantics - ucol_cloneBinary/ucol_openBinary require binary buffer
+	   for which we don't have the place to keep */
+	Collator_handlers.offset = XtOffsetOf(Collator_object, zo);
+	Collator_handlers.clone_obj = NULL;
+	Collator_handlers.dtor_obj = Collator_objects_dtor;
+	Collator_handlers.free_obj = Collator_objects_free;
 
 	/* Declare 'Collator' class properties. */
 	if( !Collator_ce_ptr )
@@ -165,19 +156,19 @@ void collator_register_Collator_class( TSRMLS_D )
  * Initialize internals of Collator_object.
  * Must be called before any other call to 'collator_object_...' functions.
  */
-void collator_object_init( Collator_object* co TSRMLS_DC )
+void collator_object_init( Collator_object* co )
 {
 	if( !co )
 		return;
 
-	intl_error_init( COLLATOR_ERROR_P( co ) TSRMLS_CC );
+	intl_error_init( COLLATOR_ERROR_P( co ) );
 }
 /* }}} */
 
 /* {{{ void collator_object_destroy( Collator_object* co )
  * Clean up mem allocted by internals of Collator_object
  */
-void collator_object_destroy( Collator_object* co TSRMLS_DC )
+void collator_object_destroy( Collator_object* co )
 {
 	if( !co )
 		return;
@@ -188,7 +179,7 @@ void collator_object_destroy( Collator_object* co TSRMLS_DC )
 		co->ucoll = NULL;
 	}
 
-	intl_error_reset( COLLATOR_ERROR_P( co ) TSRMLS_CC );
+	intl_error_reset( COLLATOR_ERROR_P( co ) );
 }
 /* }}} */
 
