@@ -5,7 +5,7 @@
  * LICENSE NOTICES
  *
  * This file is part of "streamable kanji code filter and converter",
- * which is distributed under the terms of GNU Lesser General Public 
+ * which is distributed under the terms of GNU Lesser General Public
  * License (version 2) as published by the Free Software Foundation.
  *
  * This software is distributed in the hope that it will be useful,
@@ -24,7 +24,7 @@
 /*
  * The source code included in this files was separated from mbfilter.c
  * by moriyoshi koizumi <moriyoshi@php.net> on 4 dec 2002.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,62 +32,63 @@
 #endif
 
 #include "mbfilter.h"
-#include "mbfilter_iso8859_14.h"
-#include "unicode_table_iso8859_14.h"
+#include "mbfilter_iso8859_7.h"
+#include "unicode_table_iso8859_7.h"
 
-static const char *mbfl_encoding_8859_14_aliases[] = {"ISO_8859-14", "latin8", NULL};
+static const char *mbfl_encoding_8859_7_aliases[] = {"ISO_8859-7", "greek", NULL};
 
-const mbfl_encoding mbfl_encoding_8859_14 = {
-	mbfl_no_encoding_8859_14,
-	"ISO-8859-14",
-	"ISO-8859-14",
-	(const char *(*)[])&mbfl_encoding_8859_14_aliases,
+const mbfl_encoding mbfl_encoding_8859_7 = {
+	mbfl_no_encoding_8859_7,
+	"ISO-8859-7",
+	"ISO-8859-7",
+	(const char *(*)[])&mbfl_encoding_8859_7_aliases,
 	NULL,
 	MBFL_ENCTYPE_SBCS
 };
 
-const struct mbfl_identify_vtbl vtbl_identify_8859_14 = {
-	mbfl_no_encoding_8859_14,
+const struct mbfl_identify_vtbl vtbl_identify_8859_7 = {
+	mbfl_no_encoding_8859_7,
 	mbfl_filt_ident_common_ctor,
 	mbfl_filt_ident_common_dtor,
 	mbfl_filt_ident_true
 };
 
-const struct mbfl_convert_vtbl vtbl_8859_14_wchar = {
-	mbfl_no_encoding_8859_14,
+const struct mbfl_convert_vtbl vtbl_8859_7_wchar = {
+	mbfl_no_encoding_8859_7,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_8859_14_wchar,
+	mbfl_filt_conv_8859_7_wchar,
 	mbfl_filt_conv_common_flush
 };
 
-const struct mbfl_convert_vtbl vtbl_wchar_8859_14 = {
+const struct mbfl_convert_vtbl vtbl_wchar_8859_7 = {
 	mbfl_no_encoding_wchar,
-	mbfl_no_encoding_8859_14,
+	mbfl_no_encoding_8859_7,
 	mbfl_filt_conv_common_ctor,
 	mbfl_filt_conv_common_dtor,
-	mbfl_filt_conv_wchar_8859_14,
+	mbfl_filt_conv_wchar_8859_7,
 	mbfl_filt_conv_common_flush
 };
 
+
 #define CK(statement)	do { if ((statement) < 0) return (-1); } while (0)
 
 /*
- * ISO-8859-14 => wchar
+ * ISO-8859-7 => wchar
  */
-int mbfl_filt_conv_8859_14_wchar(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_8859_7_wchar(int c, mbfl_convert_filter *filter)
 {
 	int s;
 
 	if (c >= 0 && c < 0xa0) {
 		s = c;
 	} else if (c >= 0xa0 && c < 0x100) {
-		s = iso8859_14_ucs_table[c - 0xa0];
+		s = iso8859_7_ucs_table[c - 0xa0];
 		if (s <= 0) {
 			s = c;
 			s &= MBFL_WCSPLANE_MASK;
-			s |= MBFL_WCSPLANE_8859_14;
+			s |= MBFL_WCSPLANE_8859_7;
 		}
 	} else {
 		s = c;
@@ -101,9 +102,9 @@ int mbfl_filt_conv_8859_14_wchar(int c, mbfl_convert_filter *filter)
 }
 
 /*
- * wchar => ISO-8859-14
+ * wchar => ISO-8859-7
  */
-int mbfl_filt_conv_wchar_8859_14(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_wchar_8859_7(int c, mbfl_convert_filter *filter)
 {
 	int s, n;
 
@@ -113,13 +114,13 @@ int mbfl_filt_conv_wchar_8859_14(int c, mbfl_convert_filter *filter)
 		s = -1;
 		n = 95;
 		while (n >= 0) {
-			if (c == iso8859_14_ucs_table[n]) {
+			if (c == iso8859_7_ucs_table[n]) {
 				s = 0xa0 + n;
 				break;
 			}
 			n--;
 		}
-		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_14) {
+		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_7) {
 			s = c & MBFL_WCSPLANE_MASK;
 		}
 	}
