@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2016 The PHP Group                                |
+  | Copyright (c) 1997-2015 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -27,16 +27,16 @@
 #include "ext/standard/info.h"
 #include "php_mysqli_structs.h"
 
-extern void php_mysqli_throw_sql_exception(char *sqlstate, int errorno TSRMLS_DC, char *format, ...);
+extern void php_mysqli_throw_sql_exception(char *sqlstate, int errorno, char *format, ...);
 
 /* {{{ proto bool mysqli_report(int flags)
    sets report level */
 PHP_FUNCTION(mysqli_report)
 {
-	long flags;
+	zend_long flags;
 
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &flags) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &flags) == FAILURE) {
 		return;
 	}
 
@@ -47,14 +47,14 @@ PHP_FUNCTION(mysqli_report)
 /* }}} */
 
 /* {{{ void php_mysqli_report_error(char *sqlstate, int errorno, char *error) */
-void php_mysqli_report_error(const char *sqlstate, int errorno, const char *error TSRMLS_DC)
+void php_mysqli_report_error(const char *sqlstate, int errorno, const char *error)
 {
-	php_mysqli_throw_sql_exception((char *)sqlstate, errorno TSRMLS_CC, "%s", error);
+	php_mysqli_throw_sql_exception((char *)sqlstate, errorno, "%s", error);
 }
 /* }}} */
 
 /* {{{ void php_mysqli_report_index() */
-void php_mysqli_report_index(const char *query, unsigned int status TSRMLS_DC) {
+void php_mysqli_report_index(const char *query, unsigned int status) {
 	char index[15];
 
 	if (status & SERVER_QUERY_NO_GOOD_INDEX_USED) {
@@ -64,7 +64,7 @@ void php_mysqli_report_index(const char *query, unsigned int status TSRMLS_DC) {
 	} else {
 		return;
 	}
-	php_mysqli_throw_sql_exception("00000", 0 TSRMLS_CC, "%s used in query/prepared statement %s", index, query);
+	php_mysqli_throw_sql_exception("00000", 0, "%s used in query/prepared statement %s", index, query);
 }
 /* }}} */
 
