@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend OPcache                                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2016 The PHP Group                                |
+   | Copyright (c) 1998-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -36,7 +36,7 @@ void zend_accel_hash_clean(zend_accel_hash *accel_hash)
 	memset(accel_hash->hash_table, 0, sizeof(zend_accel_hash_entry *)*accel_hash->max_num_entries);
 }
 
-void zend_accel_hash_init(zend_accel_hash *accel_hash, zend_uint hash_size)
+void zend_accel_hash_init(zend_accel_hash *accel_hash, uint32_t hash_size)
 {
 	uint i;
 
@@ -71,7 +71,7 @@ void zend_accel_hash_init(zend_accel_hash *accel_hash, zend_uint hash_size)
  * Returns pointer the actual hash entry on success
  * key needs to be already allocated as it is not copied
  */
-zend_accel_hash_entry* zend_accel_hash_update(zend_accel_hash *accel_hash, char *key, zend_uint key_length, zend_bool indirect, void *data)
+zend_accel_hash_entry* zend_accel_hash_update(zend_accel_hash *accel_hash, char *key, uint32_t key_length, zend_bool indirect, void *data)
 {
 	zend_ulong hash_value;
 	zend_ulong index;
@@ -137,27 +137,27 @@ zend_accel_hash_entry* zend_accel_hash_update(zend_accel_hash *accel_hash, char
 	return entry;
 }
 
-/* Returns the data associated with key on success
- * Returns NULL if data doesn't exist
- */
-void* zend_accel_hash_find(zend_accel_hash *accel_hash, char *key, zend_uint key_length)
+static zend_always_inline void* zend_accel_hash_find_ex(zend_accel_hash *accel_hash, char *key, uint32_t key_length, zend_ulong hash_value, int data)
 {
-	zend_ulong hash_value;
-	zend_ulong index;
-	zend_accel_hash_entry *entry;
+	zend_ulong index = hash_value % accel_hash->max_num_entries;
+	zend_accel_hash_entry *entry = accel_hash->hash_table[index];
 
-	hash_value = zend_inline_hash_func(key, key_length);
-	index = hash_value % accel_hash->max_num_entries;
-
-	entry = accel_hash->hash_table[index];
 	while (entry) {
 		if (entry->hash_value == hash_value
 			&& entry->key_length == key_length
 			&& !memcmp(entry->key, key, key_length)) {
 			if (entry->indirect) {
-				return ((zend_accel_hash_entry *) entry->data)->data;
+				if (data) {
+					return ((zend_accel_hash_entry*)entry->data)->data;
+				} else {
+					return entry->data;
+				}
 			} else {
-				return entry->data;
+				if (data) {
+					return entry->data;
+				} else {
+					return entry;
+				}
 			}
 		}
 		entry = entry->next;
@@ -165,37 +165,61 @@ void* zend_accel_hash_find(zend_accel_hash *accel_hash, char *key, zend_uint key
 	return NULL;
 }
 
+/* Returns the data associated with key on success
+ * Returns NULL if data doesn't exist
+ */
+void* zend_accel_hash_find(zend_accel_hash *accel_hash, zend_string *key)
+{
+	return zend_accel_hash_find_ex(
+		accel_hash,
+		ZSTR_VAL(key),
+		ZSTR_LEN(key),
+		zend_string_hash_val(key),
+		1);
+}
+
 /* Returns the hash entry associated with key on success
  * Returns NULL if it doesn't exist
  */
-zend_accel_hash_entry* zend_accel_hash_find_entry(zend_accel_hash *accel_hash, char *key, zend_uint key_length)
+zend_accel_hash_entry* zend_accel_hash_find_entry(zend_accel_hash *accel_hash, zend_string *key)
 {
-	zend_ulong hash_value;
-	zend_ulong index;
-	zend_accel_hash_entry *entry;
-
-	hash_value = zend_inline_hash_func(key, key_length);
-	index = hash_value % accel_hash->max_num_entries;
-
-	entry = accel_hash->hash_table[index];
-	while (entry) {
-		if (entry->hash_value == hash_value
-			&& entry->key_length == key_length
-			&& !memcmp(entry->key, key, key_length)) {
-			if (entry->indirect) {
-				return (zend_accel_hash_entry *) entry->data;
-			} else {
-				return entry;
-			}
-		}
-		entry = entry->next;
-	}
-	return NULL;
+	return (zend_accel_hash_entry *)zend_accel_hash_find_ex(
+		accel_hash,
+		ZSTR_VAL(key),
+		ZSTR_LEN(key),
+		zend_string_hash_val(key),
+		0);
 }
 
-int zend_accel_hash_unlink(zend_accel_hash *accel_hash, char *key, zend_uint key_length)
+/* Returns the data associated with key on success
+ * Returns NULL if data doesn't exist
+ */
+void* zend_accel_hash_str_find(zend_accel_hash *accel_hash, char *key, uint32_t key_length)
 {
-	zend_ulong hash_value;
+	return zend_accel_hash_find_ex(
+		accel_hash,
+		key,
+		key_length,
+		zend_inline_hash_func(key, key_length),
+		1);
+}
+
+/* Returns the hash entry associated with key on success
+ * Returns NULL if it doesn't exist
+ */
+zend_accel_hash_entry* zend_accel_hash_str_find_entry(zend_accel_hash *accel_hash, char *key, uint32_t key_length)
+{
+	return (zend_accel_hash_entry *)zend_accel_hash_find_ex(
+		accel_hash,
+		key,
+		key_length,
+		zend_inline_hash_func(key, key_length),
+		0);
+}
+
+int zend_accel_hash_unlink(zend_accel_hash *accel_hash, char *key, uint32_t key_length)
+{
+    zend_ulong hash_value;
     zend_ulong index;
     zend_accel_hash_entry *entry, *last_entry=NULL;
 
