@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend OPcache                                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2016 The PHP Group                                |
+   | Copyright (c) 1998-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -68,6 +68,7 @@
 #define FAILED_REATTACHED       2
 #define SUCCESSFULLY_REATTACHED 4
 #define ALLOC_FAIL_MAPPING      8
+#define ALLOC_FALLBACK          9
 
 typedef struct _zend_shared_segment {
     size_t  size;
@@ -102,7 +103,7 @@ typedef struct _zend_smm_shared_globals {
     /* Amount of free shared memory */
     size_t                     shared_free;
     /* Amount of shared memory allocated by garbage */
-    int                        wasted_shared_memory;
+    size_t                     wasted_shared_memory;
     /* No more shared memory flag */
     zend_bool                  memory_exhausted;
     /* Saved Shared Allocator State */
@@ -124,30 +125,32 @@ void zend_shared_alloc_shutdown(void);
 void *zend_shared_alloc(size_t size);
 
 /* copy into shared memory */
-void *_zend_shared_memdup(void *p, size_t size, zend_bool free_source TSRMLS_DC);
+void *_zend_shared_memdup(void *p, size_t size, zend_bool free_source);
 int  zend_shared_memdup_size(void *p, size_t size);
 
 typedef union _align_test {
 	void   *ptr;
 	double  dbl;
-	long    lng;
+	zend_long  lng;
 } align_test;
 
 #if ZEND_GCC_VERSION >= 2000
-# define PLATFORM_ALIGNMENT (__alignof__ (align_test))
+# define PLATFORM_ALIGNMENT (__alignof__(align_test) < 8 ? 8 : __alignof__(align_test))
 #else
 # define PLATFORM_ALIGNMENT (sizeof(align_test))
 #endif
 
 #define ZEND_ALIGNED_SIZE(size) \
-	((size + PLATFORM_ALIGNMENT - 1) & ~(PLATFORM_ALIGNMENT - 1))
+	ZEND_MM_ALIGNED_SIZE_EX(size, PLATFORM_ALIGNMENT)
 
 /* exclusive locking */
-void zend_shared_alloc_lock(TSRMLS_D);
-void zend_shared_alloc_unlock(TSRMLS_D); /* returns the allocated size during lock..unlock */
-void zend_shared_alloc_safe_unlock(TSRMLS_D);
+void zend_shared_alloc_lock(void);
+void zend_shared_alloc_unlock(void); /* returns the allocated size during lock..unlock */
+void zend_shared_alloc_safe_unlock(void);
 
 /* old/new mapping functions */
+void zend_shared_alloc_init_xlat_table(void);
+void zend_shared_alloc_destroy_xlat_table(void);
 void zend_shared_alloc_clear_xlat_table(void);
 void zend_shared_alloc_register_xlat_entry(const void *old, const void *new);
 void *zend_shared_alloc_get_xlat_entry(const void *old);
@@ -158,7 +161,7 @@ void zend_shared_alloc_restore_state(void);
 const char *zend_accel_get_shared_model(void);
 
 /* memory write protection */
-void zend_accel_shared_protect(int mode TSRMLS_DC);
+void zend_accel_shared_protect(int mode);
 
 #ifdef USE_MMAP
 extern zend_shared_memory_handlers zend_alloc_mmap_handlers;
