@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2015 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -70,7 +70,7 @@ zend_module_entry pdo_dblib_module_entry = {
 	NULL,
 	PHP_RSHUTDOWN(pdo_dblib),
 	PHP_MINFO(pdo_dblib),
-	"1.0.1",
+	PHP_PDO_DBLIB_VERSION,
 	PHP_MODULE_GLOBALS(dblib),
 	PHP_GINIT(dblib),
 	NULL,
@@ -86,23 +86,23 @@ ZEND_GET_MODULE(pdo_dblib)
 #endif
 #endif
 
-int error_handler(DBPROCESS *dbproc, int severity, int dberr,
+int pdo_dblib_error_handler(DBPROCESS *dbproc, int severity, int dberr,
 	int oserr, char *dberrstr, char *oserrstr)
 {
 	pdo_dblib_err *einfo;
 	char *state = "HY000";
-	TSRMLS_FETCH();
 
 	if(dbproc) {
 		einfo = (pdo_dblib_err*)dbgetuserdata(dbproc);
 		if (!einfo) einfo = &DBLIB_G(err);
 	} else {
 		einfo = &DBLIB_G(err);
-	}	
+	}
 
 	einfo->severity = severity;
 	einfo->oserr = oserr;
 	einfo->dberr = dberr;
+
 	if (einfo->oserrstr) {
 		efree(einfo->oserrstr);
 	}
@@ -128,20 +128,13 @@ int error_handler(DBPROCESS *dbproc, int severity, int dberr,
 	}
 	strcpy(einfo->sqlstate, state);
 
-#if 0
-	php_error_docref(NULL TSRMLS_CC, E_WARNING,
-		"dblib error: %d %s (severity %d)",
-		dberr, dberrstr, severity);	
-#endif
-
 	return INT_CANCEL;
 }
 
-int msg_handler(DBPROCESS *dbproc, DBINT msgno, int msgstate,
+int pdo_dblib_msg_handler(DBPROCESS *dbproc, DBINT msgno, int msgstate,
 	int severity, char *msgtext, char *srvname, char *procname, DBUSMALLINT line)
 {
 	pdo_dblib_err *einfo;
-	TSRMLS_FETCH();
 
 	if (severity) {
 		einfo = (pdo_dblib_err*)dbgetuserdata(dbproc);
@@ -156,10 +149,6 @@ int msg_handler(DBPROCESS *dbproc, DBINT msgno, int msgstate,
 		einfo->lastmsg = estrdup(msgtext);
 	}
 
-#if 0
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "dblib message: %s (severity %d)", msgtext, severity);
-#endif
-
 	return 0;
 }
 
@@ -191,23 +180,14 @@ PHP_MINIT_FUNCTION(pdo_dblib)
 	if (FAIL == dbinit()) {
 		return FAILURE;
 	}
-	
+
 	if (FAILURE == php_pdo_register_driver(&pdo_dblib_driver)) {
 		return FAILURE;
 	}
-	
-	/* TODO: 
-	
-	dbsetifile()
-	dbsetmaxprocs()
-	dbsetlogintime()
-	dbsettime()
-	
-	 */
 
 #if !PHP_DBLIB_IS_MSSQL
-	dberrhandle(error_handler);
-	dbmsghandle(msg_handler);
+	dberrhandle((EHANDLEFUNC) pdo_dblib_error_handler);
+	dbmsghandle((MHANDLEFUNC) pdo_dblib_msg_handler);
 #endif
 
 	return SUCCESS;
