@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -27,10 +27,11 @@
 
 /* {{{ reference_levdist
  * reference implementation, only optimized for memory usage, not speed */
-static int reference_levdist(const char *s1, int l1, const char *s2, int l2, int cost_ins, int cost_rep, int cost_del )
+static zend_long reference_levdist(const char *s1, size_t l1, const char *s2, size_t l2, zend_long cost_ins, zend_long cost_rep, zend_long cost_del )
 {
-	int *p1, *p2, *tmp;
-	int i1, i2, c0, c1, c2;
+	zend_long *p1, *p2, *tmp;
+	zend_long c0, c1, c2;
+	size_t i1, i2;
 
 	if (l1 == 0) {
 		return l2 * cost_ins;
@@ -42,8 +43,8 @@ static int reference_levdist(const char *s1, int l1, const char *s2, int l2, int
 	if ((l1 > LEVENSHTEIN_MAX_LENGTH) || (l2 > LEVENSHTEIN_MAX_LENGTH)) {
 		return -1;
 	}
-	p1 = safe_emalloc((l2 + 1), sizeof(int), 0);
-	p2 = safe_emalloc((l2 + 1), sizeof(int), 0);
+	p1 = safe_emalloc((l2 + 1), sizeof(zend_long), 0);
+	p2 = safe_emalloc((l2 + 1), sizeof(zend_long), 0);
 
 	for (i2 = 0; i2 <= l2; i2++) {
 		p1[i2] = i2 * cost_ins;
@@ -78,9 +79,9 @@ static int reference_levdist(const char *s1, int l1, const char *s2, int l2, int
 
 /* {{{ custom_levdist
  */
-static int custom_levdist(char *str1, char *str2, char *callback_name TSRMLS_DC)
+static int custom_levdist(char *str1, char *str2, char *callback_name)
 {
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "The general Levenshtein support is not there yet");
+	php_error_docref(NULL, E_WARNING, "The general Levenshtein support is not there yet");
 	/* not there yet */
 
 	return -1;
@@ -94,30 +95,30 @@ PHP_FUNCTION(levenshtein)
 	int argc = ZEND_NUM_ARGS();
 	char *str1, *str2;
 	char *callback_name;
-	int str1_len, str2_len, callback_len;
-	long cost_ins, cost_rep, cost_del;
-	int distance = -1;
+	size_t str1_len, str2_len, callback_len;
+	zend_long cost_ins, cost_rep, cost_del;
+	zend_long distance = -1;
 
 	switch (argc) {
 		case 2: /* just two strings: use maximum performance version */
-			if (zend_parse_parameters(2 TSRMLS_CC, "ss", &str1, &str1_len, &str2, &str2_len) == FAILURE) {
+			if (zend_parse_parameters(2, "ss", &str1, &str1_len, &str2, &str2_len) == FAILURE) {
 				return;
 			}
 			distance = reference_levdist(str1, str1_len, str2, str2_len, 1, 1, 1);
 			break;
 
 		case 5: /* more general version: calc cost by ins/rep/del weights */
-			if (zend_parse_parameters(5 TSRMLS_CC, "sslll", &str1, &str1_len, &str2, &str2_len, &cost_ins, &cost_rep, &cost_del) == FAILURE) {
+			if (zend_parse_parameters(5, "sslll", &str1, &str1_len, &str2, &str2_len, &cost_ins, &cost_rep, &cost_del) == FAILURE) {
 				return;
 			}
 			distance = reference_levdist(str1, str1_len, str2, str2_len, cost_ins, cost_rep, cost_del);
 			break;
 
 		case 3: /* most general version: calc cost by user-supplied function */
-			if (zend_parse_parameters(3 TSRMLS_CC, "sss", &str1, &str1_len, &str2, &str2_len, &callback_name, &callback_len) == FAILURE) {
+			if (zend_parse_parameters(3, "sss", &str1, &str1_len, &str2, &str2_len, &callback_name, &callback_len) == FAILURE) {
 				return;
 			}
-			distance = custom_levdist(str1, str2, callback_name TSRMLS_CC);
+			distance = custom_levdist(str1, str2, callback_name);
 			break;
 
 		default:
@@ -125,7 +126,7 @@ PHP_FUNCTION(levenshtein)
 	}
 
 	if (distance < 0 && /* TODO */ ZEND_NUM_ARGS() != 3) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Argument string(s) too long");
+		php_error_docref(NULL, E_WARNING, "Argument string(s) too long");
 	}
 
 	RETURN_LONG(distance);
