@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2015 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -24,6 +24,9 @@
 extern zend_module_entry xsl_module_entry;
 #define phpext_xsl_ptr &xsl_module_entry
 
+#include "php_version.h"
+#define PHP_XSL_VERSION PHP_VERSION
+
 #ifdef ZTS
 #include "TSRM.h"
 #endif
@@ -32,7 +35,7 @@ extern zend_module_entry xsl_module_entry;
 #include <libxslt/xsltInternals.h>
 #include <libxslt/xsltutils.h>
 #include <libxslt/transform.h>
-#include <libxslt/security.h> 
+#include <libxslt/security.h>
 #if HAVE_XSL_EXSLT
 #include <libexslt/exslt.h>
 #include <libexslt/exsltconfig.h>
@@ -54,10 +57,9 @@ extern zend_module_entry xsl_module_entry;
 #define XSL_SECPREF_DEFAULT 44
 
 typedef struct _xsl_object {
-	zend_object  std;
 	void *ptr;
 	HashTable *prop_handler;
-	zend_object_handle handle;
+	zval handle;
 	HashTable *parameter;
 	int hasKeys;
 	int registerPhpFunctions;
@@ -65,13 +67,20 @@ typedef struct _xsl_object {
 	HashTable *node_list;
 	php_libxml_node_object *doc;
 	char *profiling;
-	long securityPrefs;
+	zend_long securityPrefs;
 	int securityPrefsSet;
+	zend_object  std;
 } xsl_object;
 
-void php_xsl_set_object(zval *wrapper, void *obj TSRMLS_DC);
-void xsl_objects_free_storage(void *object TSRMLS_DC);
-zval *php_xsl_create_object(xsltStylesheetPtr obj, int *found, zval *wrapper_in, zval *return_value  TSRMLS_DC);
+static inline xsl_object *php_xsl_fetch_object(zend_object *obj) {
+	return (xsl_object *)((char*)(obj) - XtOffsetOf(xsl_object, std));
+}
+
+#define Z_XSL_P(zv) php_xsl_fetch_object(Z_OBJ_P((zv)))
+
+void php_xsl_set_object(zval *wrapper, void *obj);
+void xsl_objects_free_storage(zend_object *object);
+void php_xsl_create_object(xsltStylesheetPtr obj, zval *wrapper_in, zval *return_value );
 
 void xsl_ext_function_string_php(xmlXPathParserContextPtr ctxt, int nargs);
 void xsl_ext_function_object_php(xmlXPathParserContextPtr ctxt, int nargs);
@@ -79,16 +88,16 @@ void xsl_ext_function_object_php(xmlXPathParserContextPtr ctxt, int nargs);
 #define REGISTER_XSL_CLASS(ce, name, parent_ce, funcs, entry) \
 INIT_CLASS_ENTRY(ce, name, funcs); \
 ce.create_object = xsl_objects_new; \
-entry = zend_register_internal_class_ex(&ce, parent_ce, NULL TSRMLS_CC);
+entry = zend_register_internal_class_ex(&ce, parent_ce);
 
 #define XSL_DOMOBJ_NEW(zval, obj, ret) \
-	if (NULL == (zval = php_xsl_create_object(obj, ret, zval, return_value TSRMLS_CC))) { \
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create required DOM object"); \
+	zval = php_xsl_create_object(obj, ret, zval, return_value); \
+	if (ZVAL_IS_NULL(zval)) { \
+		php_error_docref(NULL, E_WARNING, "Cannot create required DOM object"); \
 		RETURN_FALSE; \
 	}
 
 
-
 PHP_MINIT_FUNCTION(xsl);
 PHP_MSHUTDOWN_FUNCTION(xsl);
 PHP_RINIT_FUNCTION(xsl);
@@ -96,9 +105,9 @@ PHP_RSHUTDOWN_FUNCTION(xsl);
 PHP_MINFO_FUNCTION(xsl);
 
 
-/* 
+/*
   	Declare any global variables you may need between the BEGIN
-	and END macros here:     
+	and END macros here:
 
 ZEND_BEGIN_MODULE_GLOBALS(xsl)
 	long  global_value;
@@ -106,12 +115,12 @@ ZEND_BEGIN_MODULE_GLOBALS(xsl)
 ZEND_END_MODULE_GLOBALS(xsl)
 */
 
-/* In every utility function you add that needs to use variables 
-   in php_xsl_globals, call TSRM_FETCH(); after declaring other 
-   variables used by that function, or better yet, pass in TSRMLS_CC
+/* In every utility function you add that needs to use variables
+   in php_xsl_globals, call TSRM_FETCH(); after declaring other
+   variables used by that function, or better yet, pass in
    after the last function argument and declare your utility function
-   with TSRMLS_DC after the last declared argument.  Always refer to
-   the globals in your function as XSL_G(variable).  You are 
+   with after the last declared argument.  Always refer to
+   the globals in your function as XSL_G(variable).  You are
    encouraged to rename these macros something shorter, see
    examples in any other php module directory.
 */
